// Copyright 2015, 2016 Ethcore (UK) Ltd.
// This file is part of Parity.

// Parity is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Parity is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with Parity.  If not, see <http://www.gnu.org/licenses/>.

import BigNumber from 'bignumber.js';
import { action, computed, observable } from 'mobx';

import Contracts from '../../contracts';
import { hashToImageUrl } from '../../redux/util';

const builtinApps = [
  {
    id: '0xf9f2d620c2e08f83e45555247146c62185e4ab7cf82a4b9002a265a0d020348f',
    url: 'basiccoin',
    name: 'Token Deployment',
    description: 'Deploy new basic tokens that you are able to send around',
    author: 'Parity Team <admin@ethcore.io>',
    version: '1.0.0'
  },
  {
    id: '0xd1adaede68d344519025e2ff574650cd99d3830fe6d274c7a7843cdc00e17938',
    url: 'registry',
    name: 'Registry',
    description: 'A global registry of addresses on the network',
    author: 'Parity Team <admin@ethcore.io>',
    version: '1.0.0'
  },
  {
    id: '0x0a8048117e51e964628d0f2d26342b3cd915248b59bcce2721e1d05f5cfa2208',
    url: 'tokenreg',
    name: 'Token Registry',
    description: 'A registry of transactable tokens on the network',
    author: 'Parity Team <admin@ethcore.io>',
    version: '1.0.0'
  },
  {
    id: '0xf49089046f53f5d2e5f3513c1c32f5ff57d986e46309a42d2b249070e4e72c46',
    url: 'signaturereg',
    name: 'Method Registry',
    description: 'A registry of method signatures for lookups on transactions',
    author: 'Parity Team <admin@ethcore.io>',
    version: '1.0.0'
  },
  {
    id: '0x058740ee9a5a3fb9f1cfa10752baec87e09cc45cd7027fd54708271aca300c75',
    url: 'githubhint',
    name: 'GitHub Hint',
    description: 'A mapping of GitHub URLs to hashes for use in contracts as references',
    author: 'Parity Team <admin@ethcore.io>',
    version: '1.0.0'
  }
];

// TODO: This is just since we are moving gavcoin to its own repo, for a proper network solution
// we need to pull the network apps from the dapp registry. (Builtins & local apps unaffected)
// TODO: Manifest needs to be pulled from the content as well, however since the content may or may
// not be available locally (and refreshes work for index, and will give a 503), we are putting it
// in here. This part needs to be cleaned up.
const networkApps = [
  {
    id: '0xd798a48831b4ccdbc71de206a1d6a4aa73546c7b6f59c22a47452af414dc64d6',
    name: 'GAVcoin',
    description: 'Manage your GAVcoins, the hottest new property in crypto',
    author: 'Gavin Wood',
    version: '1.0.0'
  }
];

export default class DappsStore {
  @observable apps = [];
  @observable hidden = [];
  @observable modalOpen = false;

  constructor (api) {
    this._api = api;

    this._readHiddenApps();
    this._fetch();
  }

  @computed get visible () {
    return this.apps.filter((app) => !this.hidden.includes(app.id));
  }

  @action openModal = () => {
    this.modalOpen = true;
  }

  @action closeModal = () => {
    this.modalOpen = false;
  }

  @action hideApp = (id) => {
    this.hidden = this.hidden.concat(id);
    this._writeHiddenApps();
  }

  @action showApp = (id) => {
    this.hidden = this.hidden.filter((_id) => _id !== id);
    this._writeHiddenApps();
  }

  _getHost (api) {
    return process.env.NODE_ENV === 'production'
      ? this._api.dappsUrl
      : '';
  }

  @action _fetch () {
    const { dappReg } = Contracts.get();

    return fetch(`${this._getHost()}/api/apps`)
      .then((response) => {
        return response.ok
          ? response.json()
          : [];
      })
      .catch((error) => {
        console.warn('DappsStore:fetch', error);
        return [];
      })
      .then((_localApps) => {
        const localApps = _localApps
          .filter((app) => !['ui'].includes(app.id))
          .map((app) => {
            app.type = 'local';
            return app;
          });

        return this._api.parity
          .registryAddress()
          .then((registryAddress) => {
            if (new BigNumber(registryAddress).eq(0)) {
              return [];
            }

            const _builtinApps = builtinApps
              .map((app) => {
                app.type = 'builtin';
                return app;
              });

            return networkApps
              .map((app) => {
                app.type = 'network';
                return app;
              })
              .concat(_builtinApps);
          })
          .then((registryApps) => {
            return registryApps
              .concat(localApps)
              .sort((a, b) => (a.name || '').localeCompare(b.name || ''));
          });
      })
      .then((apps) => {
        return Promise
          .all([
            Promise.all(apps.map((app) => dappReg.getImage(app.id))),
            Promise.all(apps.map((app) => dappReg.getContent(app.id)))
          ])
          .then(([images, content]) => {
            this.apps = apps.map((app, index) => {
              return Object.assign(app, {
                image: hashToImageUrl(images[index]),
                contentHash: this._api.util.bytesToHex(content[index]).substr(2)
              });
            });
          });
      })
      .catch((error) => {
        console.warn('DappsStore:fetch', error);
      });
  }

  _manifest (app, contentHash) {
    fetch(`${this.getHost()}/${contentHash}/manifest.json`)
      .then((response) => {
        return response.ok
          ? response.json()
          : {};
      })
      .then((manifest) => {
        Object.keys.forEach((key) => {
          app[key] = manifest[key];
        });

        return app;
      })
      .catch((error) => {
        console.warn('DappsStore:manifest', error);
      });
  }

  @action _readHiddenApps () {
    const stored = localStorage.getItem('hiddenApps');

    if (stored) {
      try {
        this.hidden = JSON.parse(stored);
      } catch (error) {
        console.warn('DappsStore:readHiddenApps', error);
      }
    }
  }

  _writeHiddenApps () {
    localStorage.setItem('hiddenApps', JSON.stringify(this.hidden));
  }
}
